<?php
namespace Models;

// Модель для работы с Фискальным регистратором
class Fr extends \DB\SQL\Mapper{
	public $db, $f3, $web, $sets, $method='POST', $messeges, $ticket_type=array('1'=>0, '2'=>1);

	public function __construct($db, $f3) {
		parent::__construct($db, 'fr_settings' ); // подключаемся к таблице с настройками
		$this->f3=$f3;
		$this->db=$db;
		$this->web=\Web::instance();
		$this->messeges=new \Models\Messeges();
		$this->sets=$this->get_fr();
	}

	// Настройки ФР
	function get_fr() {
		$this->reset();
		$this->load();
		$sets=$this->cast();
		$sets['pays_type']=json_decode($sets['pays_type'], true);
		if(!$sets['vat_type']) {
			$sets['vat_type']=array(
				'0'=>0,
				'1'=>1,
				'2'=>2,
				'3'=>3,
				'4'=>4,
				'5'=>5
			);
		}
		else {
			$sets['vat_type']=json_decode($sets['vat_type'], true);
		}
		return $sets;
	}

	// Сохраняем настройки ФР
	function save_fr($post) {
		$post['pays_type']=json_encode($post['pays']);
		$post['vat_type']=json_encode($post['vat']);
		unset($post['pays'], $post['vat']);
		$this->reset();
		$this->get_fr();
		$this->copyFrom($post);
		$this->save();
		if($this->dry()) {
			return array(
				'success'=>false,
				'txt'=>'db_error'
			);
		}
		return array(
			'success'=>true,
			'txt'=>'save_successful'
		);
	}

	// Отправка запроса
	function send_ajax($data) {
		if(!$this->sets['active']) return true;
		$url=$this->sets['url'];
		$vars=array(
			'data'=>$data
		);
		$options = array(
			'timeout'=>3,
    		'method'  => $this->method,
    		'content' => 'data='.$data
		);
		$res=$this->web->request($url, $options);
		//var_dump($res);
		// Проверяем ответ сревера
		if(!mb_strpos($res['headers'][0], '200 OK')) {
			if($res['error']) {
				echo $res['error'];
			}
			else {
				echo $res['body'];
			}
			exit();
		}
		// проверяем смог сделать сервер
		$ok=true;
		$er='';
		$strs=explode("\n", $res['body']);
		$strs = array_diff($strs, array(''));
		$d=explode("\n", $data);
		$d = array_diff($d, array(''));
		foreach ($strs as $str) {
			$csv=str_getcsv($str, ';');
			if($csv[1]!=0) {
				$ok=false;
				$er.='<p class="false">Ошибка '.$csv[1].' '.$csv[2].'</p>';
			}
		}
		if($ok && count($strs)==count($d)) {
			return true;
		}
		else {
			if($er=='') {
				echo '<pre>'.$data.'</pre>';
				echo'<pre>'.$res['body'].'</pre>';
				$er='<p class="false">'.$this->messeges->messege('error')['txt'].'</p>';
			}
			echo $er;
			exit();
		}
	}

	// Открытие смены
	function open_session() {
		$strs="open_session;\n";
		if(!$this->sets['type']) {
			if($this->sets['header']) {
				$strs.=$this->sets['header']."\n";
			}
			$strs.="p;\n";
			$strs.="p;ОТКРЫТИЕ СМЕНЫ;\n";
			$strs.="p;".date("d-m-Y H:i").";\n";
			$strs.="p;\n";
			$strs.="p;Кассир ".$this->f3->get('SESSION.user.short_name').";\n";
			$strs.="p;\n";
			if($this->sets['footer']) {
				$strs.=$this->sets['footer']."\n";
			}
		}
		$this->send_ajax($strs);
	}

	// Закрытие смены
	function z($change) {
		$strs="z;\n";
		if(!$this->sets['type']) {
			$strs.=$this->report($change, 'z');
		}
		$this->send_ajax($strs);
	}

	// X-отчет смены
	function x($change) {
		$strs="x;\n";
		if(!$this->sets['type']) {
			$strs.=$this->report($change, 'x');
		}
		$this->send_ajax($strs);
	}

	function report($change, $type) {
		$types=array('x'=>'НУЛЕВОЙ ОТЧЕТ', 'z'=>'ОТЧЕТ С ГАШЕНИЕМ');
		$receipts=new \Models\Receipts($this->db, $this->f3);
		$report=$receipts->change_statistic($change['id']);
		$p=new \Models\Paymentmethods($this->db);
		$paymentmethods=$p->select_method();
		foreach ($paymentmethods as $p) {
			$pays[$p['id']]=array('title'=>$p['title'], 'cash'=>$p['cash']);
		}
		$strs='';
		if($this->sets['header']) {
			$strs.=$this->sets['header']."\n";
		}
		$strs.="p;".$types[$type].";\n";
		$strs.="p;\n";
		$strs.="p;".date("d/m/Y H:i").";\n";
		$strs.="p;Смена ".$change['id'].";\n";
		$strs.="p;Начало смены ".date("d/m/Y H:i", $change['time_start']).";\n";
		if($type=='z') {
			$strs.="p;Конец смены ".date("d/m/Y H:i").";\n";
		}
		$strs.="p;\n";
		$types=array(
			'1'=>'Продажи',
			'2'=>'Возвраты',
			'-1'=>'Внесение',
			'-2'=>'Изъятие'
		);
		foreach ($report as $type => $r) {
			$strs.="p;".$types[$type]." = ".$r['count'].";\n";
			foreach ($pays as $i => $p) {
				$strs.="p;  ".$p['title']." = ".number_format(abs($r['pay'][$i]), 2, '.', '').";\n";
				if($p['cash']==1) {
					if($type==2)$total-=$r['pay'][$i];
					else $total+=$r['pay'][$i];
				}
			}
			$strs.="p;  Сумма = ".number_format(abs($r['sum']), 2, '.', '').";\n";
			$strs.="p;\n";
		}
		$strs.="p;Наличность в кассе = ".number_format($total, 2, '.', '')."\n";
		$strs.="p;\n";
		if($this->sets['footer']) {
			$strs.=$this->sets['footer']."\n";
		}
		return $strs;
	}

	// Добавляем чек
	function ticket($t, $lines, $discount=0) {
		function replace_txt($txt) {
			$txt=str_replace(';', '', $txt);
			$txt=str_replace('"', '', $txt);
			$txt=str_replace("'", '', $txt);
			return $txt;
		}
		$user=$this->f3->get('SESSION.user.short_name');
		$vat_type=$this->f3->get('vat_type');
		$section=$this->f3->get('section');
		$strs='b;'.($this->ticket_type[$t['type']]).';Кассир '.$user.";\n"; // Открываем чек

		if($t['client']) {
			$strs.="set_tlv;1008;5;".$t['client'].";\n";
		}

		// Заголовок
		if($this->sets['header']) {
			$strs.=$this->sets['header']."\n";
		}

		// Товары

		foreach ($lines as $l) {
			if(!$l['discount'])$l['discount']=$discount;
			$title=mb_substr($l['short_title'], 0, 100);
			$title=replace_txt($title);
			$price=$l['price']+($l['price']*($l['vat']/100))*max(0, $l['vat_type']);
			$price-=$price*($l['discount']/100);
			$price*=100;
			$price=round($price);
			$vt=$vat_type;
			if($l['vat_type']==0) $vt=$this->sets['vat_type'][0];
			elseif($l['vat_type']!=0 && $l['vat']==0) $vt=$this->sets['vat_type'][1];
			elseif($l['vat_type']==1 && $l['vat']>0 && $l['vat']<=10) $vt=$this->sets['vat_type'][2];
			elseif($l['vat_type']==-1 && $l['vat']>0 && $l['vat']<=10) $vt=$this->sets['vat_type'][3];
			elseif($l['vat_type']==1 && $l['vat']>10) $vt=$this->sets['vat_type'][4];
			elseif($l['vat_type']==-1 && $l['vat']>10) $vt=$this->sets['vat_type'][5];
			$strs.='smde;'.$title.';'.$price.';'.round($l['count']*1000).';'.$vt.';'.$section.";\n";
		}
		

		// оплата
		$strs.='tmde;';
		$pays=json_decode($t['payment'], true);
		if(count($pays)==1) {
			$strs.=round($pays[0]['sum']*100).';'.max(0, $this->sets['pays_type'][$pays[0]['method']]).';';
		}
		else {
			foreach ($pays as $p) {
				$mpay[$this->sets['pays_type'][$p['method']]]=$p['sum'];
			}

			for($i=0; $i<4; $i++) {
				$strs.=max(round($mpay[$i]*100), 0).';';
			}
			
			for($i=0; $i<4; $i++) {
				$strs.=$this->sets['sno'].';';
			}
		}
		$strs.="\n";

		// подвал
		if($this->sets['footer']) {
			$strs.=$this->sets['footer']."\n";
		}
		$this->send_ajax($strs);
	}

	// Внесение в кассу
	function cashin($change, $sum) {
		$strs="imde;".$sum.";";
		// Заголовок
		if($this->sets['header']) {
			$strs.=$this->sets['header']."\n";
		}
		if($sum>=0) {
			$strs.="p;ВНЕСЕНИЕ В КАССУ;\n";
		}
		else {
			$strs.="p;ИЗЪЯТИЕ ИЗ КАССЫ;\n";
		}
		$strs.="p;\n";
		$strs.="p;Сумма = ".number_format(abs($sum), 2, '.', '').";\n";
		$strs.="p;Кассир ".$this->f3->get('SESSION.user.short_name').";\n";
		$strs.="p;Смена ".$change.";\n";
		$strs.="p;".date("d-m-Y H:i").";\n";
		$strs.="p;\n";

		// подвал
		if($this->sets['footer']) {
			$strs.=$this->sets['footer']."\n";
		}

		$this->send_ajax($strs);
	}

	// Обнуление не пробитого чека
	function reset_ticket() {
		$strs="g;";
		$this->send_ajax($strs);
	}

	// Запрос полного статуса ККМ
	function kkm_status() {
		$strs="d;";
		$this->send_ajax($strs);
	}
	
	// Продолжить печать, после аварийной остановки
	function continue_print() {
		$strs="continue_print;";
		$this->send_ajax($strs);
		return array(
			'success'=>true,
			'txt'=>'print_continue'
		);
	}
}